package main.java.SignalPlanRecommender;

import main.java.SignalPlanRecommender.ProblemDescription.DayOfWeek;
import main.java.SignalPlanRecommender.ProblemDescription.TimeOfDay;
import jcolibri.cbrcore.CaseComponent;

public class SignalPlanCalculator {
	ProblemSolution problemSolution;
	ProblemDescription problemDescription;
	SignalPlan signalPlan;
	TrafficSituation trafficSituation;
	
	public SignalPlanCalculator(CaseComponent solution, CaseComponent problem) {
		problemSolution = (ProblemSolution)solution;
		problemDescription = (ProblemDescription)problem;
		trafficSituation = problemSolution.trafficSituation;
	}
	
	public void calculateSignalPlanUsingWebster() {
		/*
		 * This method calculates the signal plan using Webster's 
		 * algorithm. Whether or not the resulting plan should be 
		 * restricted to a given cycle is determined by altering a
		 * line bellow.
		 * 
		 * In addition to use Webster, it also does some changes to 
		 * the signal plan, based on if the event is a football match
		 * etc.
		 */
		float maxPhase1, maxPhase2, maxPhase3, y, lengthPhase1, lengthPhase2, lengthPhase3, cycleTime;
		int lenghtOfYellowTime = 0;
		int[] yellowTimes;
		int[] lengthOfPhases;
		int numberOfPhases;
		maxPhase1 = (this.trafficSituation.flowNS > this.trafficSituation.flowSN) 
				? this.trafficSituation.flowNS : this.trafficSituation.flowSN;
		maxPhase2 = (this.trafficSituation.flowNE > this.trafficSituation.flowSW) 
				? this.trafficSituation.flowNE : this.trafficSituation.flowSW;
		maxPhase3 = (this.trafficSituation.flowEW > this.trafficSituation.flowWE)
				? this.trafficSituation.flowEW : this.trafficSituation.flowWE;

		//A Saturday
		if (this.problemDescription.dayOfWeek == ProblemDescription.DayOfWeek.Saturday) {
			
			if (this.problemDescription.timeOfDay == ProblemDescription.TimeOfDay.Noon) {
				System.out.println("Saturday at Noon");
				yellowTimes = new int[]{4,13,7};
				numberOfPhases = 3;
			}
			
			else {
				System.out.println("Saturday at other time");
				yellowTimes = new int[]{16,7};
				numberOfPhases = 2;
			}
		}
		//A Sunday
		else if(this.problemDescription.dayOfWeek == ProblemDescription.DayOfWeek.Sunday) {
			System.out.println("Sunday");
			yellowTimes = new int[]{16,7};
			numberOfPhases = 2;
		}
		//A weekday
		else {
			if ((this.problemDescription.timeOfDay == ProblemDescription.TimeOfDay.Morning) ||
				(this.problemDescription.timeOfDay == ProblemDescription.TimeOfDay.Afternoon))	{
				System.out.println("Weekday at morning or afternoon");
				yellowTimes = new int[]{4,13,7};
				numberOfPhases = 3;
			}
			
			else {
				System.out.println("Weekday at other time");
				yellowTimes = new int[]{16,7};
				numberOfPhases = 2;
			}
			
		}
		//Calculate the average length for use with Webster's formula

		for (int i : yellowTimes)
			lenghtOfYellowTime += i;
		lenghtOfYellowTime/= yellowTimes.length;	
		System.out.println("Length of yellowtimne" + lenghtOfYellowTime);
		
		if (numberOfPhases == 2) 
			y = maxPhase1/1800 + maxPhase3/1800;
		else 
			y = maxPhase1/1800 + maxPhase2/1800 + maxPhase3/1800;

		
		cycleTime = (float) ((1.5*(lenghtOfYellowTime*2)+5)/(1-y));
		cycleTime = round(cycleTime,5);
		if (numberOfPhases == 2){
			lengthPhase1 = ((maxPhase1/1800)/y)* (cycleTime-lenghtOfYellowTime*2);
			System.out.println("lengthofPhase1 " + lengthPhase1);
			lengthPhase2 = (float) Math.ceil(((maxPhase2/1800)/y)* (cycleTime-lenghtOfYellowTime*2));
			System.out.println("lengtofPhase2" + lengthPhase2);
		
			
			if (problemDescription.event == ProblemDescription.Event.Football && maxPhase3 > 100) {
				lengthPhase1 -= maxPhase3/18;
				lengthPhase2 += maxPhase3/18;
			}
			//If really low traffic flow, give time for just one car to pass
			if (maxPhase2 < 50) {
				while (lengthPhase2 < 3) {
					lengthPhase2 ++;
					lengthPhase1 --;
				}
			}
			
			else {
				while (lengthPhase2 < 4) {
					lengthPhase2 ++;
					lengthPhase1 --;
				}
			}
			lengthOfPhases = new int[]{(int)lengthPhase1,(int)lengthPhase2};
		}

		else  {
			lengthPhase1 = ((maxPhase1/1800)/y)* (cycleTime-lenghtOfYellowTime*3);
			System.out.println("lengthofPhase1 " + lengthPhase1);
			lengthPhase2 = (float) Math.ceil(((maxPhase2/1800)/y)* (cycleTime-lenghtOfYellowTime*3));
			System.out.println("lengtofPhase2" + lengthPhase2);
			lengthPhase3 = (float) Math.ceil(((maxPhase3/1800)/y)* (cycleTime-lenghtOfYellowTime*3));
			System.out.println("lengtofPhase3" + lengthPhase3);
			
			lengthOfPhases = new int[]{(int)lengthPhase1,(int)lengthPhase2,(int)lengthPhase3};
		}
		
		signalPlan = new SignalPlan(lengthOfPhases,yellowTimes);
		
		/*
		 * The following line can be commented or uncommented in 
		 * order to restrict the signal plan to a given cycle time
		 */
//		signalPlan.scaleToCycleOfGivenLength(60);
		
		System.out.println("SignalPlan " + this.signalPlan);
		
	}
	public int round(double i, int v)  {
		return (int) (Math.round(i/v)*v);
	}
}
