package main.java.evaluate;

import java.util.Vector;

import main.java.SignalPlanRecommender.ProblemDescription;
import main.java.SignalPlanRecommender.ProblemSolution;
import main.java.SignalPlanRecommender.SignalPlanRecommender;
import jcolibri.cbrcore.Attribute;
import jcolibri.evaluation.Evaluator;
import jcolibri.evaluation.evaluators.*;
import jcolibri.method.retrieve.NNretrieval.similarity.global.Average;
import jcolibri.method.retrieve.NNretrieval.similarity.local.Equal;
import jcolibri.method.retrieve.NNretrieval.similarity.local.Table;

public class CaseBaseEvaluator {

	SignalPlanRecommender signalPlanRecommender;
	LeaveOneOutEvaluator leaveOneOut;
	
	/**
	 * @param args
	 */
	public CaseBaseEvaluator() {
		signalPlanRecommender = new SignalPlanRecommender();
		leaveOneOut = new main.java.evaluate.LeaveOneOutEvaluator();
		leaveOneOut.init(signalPlanRecommender);

	}
	/**
	 * Does a whole cycle of leave one out evaluation.
	 * @return the average similarity of the selected 
	 * solution compared to the real solution, divided 
	 * by number of similarity values
	 * This is used as fitness in the evolutionary algorithm
	 */
	public synchronized Vector<Double> evaluateUsingLeaveOneOutEvaluator() {
		leaveOneOut.LeaveOneOut();
		return leaveOneOut.getEvaluationReport().getSeries("Similarity");
	}
	
	public synchronized double getFitnessOfCaseBaseUsingLeaveOneOut() {
		String[] allSimilarities;
		double accumulatedSimilarity = 0.0;
		double averageSimilarity = 0.0;
		Vector vector = evaluateUsingLeaveOneOutEvaluator();

		allSimilarities= vector.toString().substring(1, vector.toString().length()-1).split(",");

		for (int i=0; i<allSimilarities.length;i++) {
			accumulatedSimilarity+= Double.parseDouble(allSimilarities[i]);
		}
		averageSimilarity = accumulatedSimilarity / allSimilarities.length;
		
//		System.out.println("Average similarity = " + averageSimilarity);
		return averageSimilarity;

	}

	//Evaluates the similarity of solutions and returns a double ranged between 0.0 and 1.0
	public synchronized static double evaluateSolutionSimilarity(ProblemSolution sol1, ProblemSolution sol2) {
//		System.out.println("Sol1 " + sol1 + " Sol2 " + sol2);
		String[] solution1List, solution2List;
		solution1List = sol1.getTrafficSituation().getListOfFlows().replace(" ", "").split(",");
		solution2List = sol2.getTrafficSituation().getListOfFlows().replace(" ", "").split(",");
		
		double similarity = 0.0;
		double laneFlow1,laneFlow2, valueWeight;
		int totalTrafficFlow1, totalTrafficFlow2;
		if (solution1List.length == solution2List.length) {
			totalTrafficFlow1 = getTotalTrafficFlow(solution1List);
			totalTrafficFlow2 = getTotalTrafficFlow(solution2List);
			for(int i = 0; i<solution1List.length; i++ ) {
				laneFlow1 = Double.parseDouble(solution1List[i]);
				laneFlow2 = Double.parseDouble(solution2List[i]);
//				System.out.println("value1: "+ laneFlow1 + "value 2 " + laneFlow2 + "similarity " + similarity);
				
				//To avoid both zero division and no priority in the given lane
				if (laneFlow1 == 0) laneFlow1 = 1;
				if (laneFlow2 == 0) laneFlow2 = 1;
				
				valueWeight = (laneFlow1/totalTrafficFlow1+laneFlow2/totalTrafficFlow2)/2;
				
				if (laneFlow1 < laneFlow2) {
					similarity += laneFlow1/laneFlow2 * valueWeight;
				}
				else if (laneFlow1 >= laneFlow2) {
					similarity += laneFlow2/laneFlow1 * valueWeight;
				}
				
			}
//			System.out.println("SOLSIM = " + similarity);
		}
		
		return similarity;
	}
	
	public static int getTotalTrafficFlow(String[] trafficFlow) {
		int totalFlow = 0;
		for (int i=0;i<trafficFlow.length;i++) {
			totalFlow += Integer.parseInt(trafficFlow[i]);
		}
		return totalFlow;
	}
	
	public synchronized void setWeights(double[] weights) {
		this.signalPlanRecommender.setWeights(weights);
	}

	
	public static void main(String[] args) {
		/*
		 * Run this method in order to evaluate the case base using
		 * given feature weights.
		 */
		
		double[] weights087 = {0.031496062992125984, 0.9606299212598425, 0.08661417322834646, 0.11023622047244094, 0.007874015748031496, 0.023622047244094488, 0.007874015748031496};
		double[] weights0873 = {0.18110236220472442, 0.9921259842519685, 0.2677165354330709, 0.3228346456692913, 0.5196850393700787, 0.031496062992125984, 0.031496062992125984};
		double[] weights08734 = {0.03937007874015748, 0.984251968503937, 0.29133858267716534, 0.2125984251968504, 0.7322834645669292, 0.007874015748031496, 0.007874015748031496};
		double[] weights55_0877 = {0.2204724409448819, 0.937007874015748, 0.2677165354330709, 0.36220472440944884, 0.5118110236220472, 0.2125984251968504, 0.09448818897637795};
		double[] manualWeights = {0.1,0.7,0.95,0.7,0.1,0.8,0.3};

		CaseBaseEvaluator cbe = new CaseBaseEvaluator();
		cbe.setWeights(weights55_0877);
//		System.out.println(cbe.evaluateUsingLeaveOneOutEvaluator());
		System.out.println(cbe.getFitnessOfCaseBaseUsingLeaveOneOut());
		
		jcolibri.evaluation.tools.EvaluationResultGUI.show(Evaluator.getEvaluationReport(), "title", false);		
//		System.exit(0);
	}
	public SignalPlanRecommender getSignalPlanRecommender() {
		return signalPlanRecommender;
	}
	public void setSignalPlanRecommender(SignalPlanRecommender signalPlanRecommender) {
		this.signalPlanRecommender = signalPlanRecommender;
	}
	
}
