package main.java.genetic_algorithm;

import java.util.LinkedList;
import java.util.List;
import java.util.Random;

import main.java.evaluate.CaseBaseEvaluator;

import org.uncommons.maths.binary.BitString;
import org.uncommons.maths.random.MersenneTwisterRNG;
import org.uncommons.maths.random.Probability;
import org.uncommons.watchmaker.framework.CandidateFactory;
import org.uncommons.watchmaker.framework.EvolutionEngine;
import org.uncommons.watchmaker.framework.EvolutionObserver;
import org.uncommons.watchmaker.framework.EvolutionaryOperator;
import org.uncommons.watchmaker.framework.FitnessEvaluator;
import org.uncommons.watchmaker.framework.GenerationalEvolutionEngine;
import org.uncommons.watchmaker.framework.PopulationData;
import org.uncommons.watchmaker.framework.SelectionStrategy;
import org.uncommons.watchmaker.framework.factories.BitStringFactory;
import org.uncommons.watchmaker.framework.operators.BitStringCrossover;
import org.uncommons.watchmaker.framework.operators.BitStringMutation;
import org.uncommons.watchmaker.framework.operators.DoubleArrayCrossover;
import org.uncommons.watchmaker.framework.operators.EvolutionPipeline;
import org.uncommons.watchmaker.framework.operators.Replacement;
import org.uncommons.watchmaker.framework.selection.RouletteWheelSelection;
import org.uncommons.watchmaker.framework.termination.TargetFitness;

public class BinaryFeatureWeighter {

	/**
	 * @param args
	 */

		static CaseBaseEvaluator cbe; 
		
	public static void main(String[] args) {

		cbe = new CaseBaseEvaluator();
		Random rng = new MersenneTwisterRNG();
		CandidateFactory<BitString> factory = new BitStringFactory(49);
		List<EvolutionaryOperator<BitString>> operators = new LinkedList<EvolutionaryOperator<BitString>>();
		operators.add(new BitStringCrossover());
//			operators.add(new Replacement<double[]>(factory, new Probability(0.2)));
		operators.add(new BitStringMutation(new Probability(0.05)));
		operators.add(new Replacement<BitString>(factory, new Probability(0.05)));
		EvolutionaryOperator<BitString> pipeline = new EvolutionPipeline<BitString>(operators);
		FitnessEvaluator<BitString> fitnessEvaluator = new BinaryFeatureWeightEvaluator(cbe);
		SelectionStrategy<Object> selectionStrategy = new RouletteWheelSelection();
		

		EvolutionEngine<BitString> engine = new GenerationalEvolutionEngine<BitString>(
				factory, pipeline, fitnessEvaluator, selectionStrategy, rng);

		
		engine.addEvolutionObserver(new EvolutionObserver<BitString>() {
			public void populationUpdate(PopulationData<? extends BitString> data)
			{
				double[] bestCandidate = makeWeightsFromBitString(data.getBestCandidate());
				
				System.out.printf("Generation %d: %s, %s, %s, %s, %s, %s, %s Fitness: %s\n",
						data.getGenerationNumber(),
						bestCandidate[0],
						bestCandidate[1],
						bestCandidate[2],
						bestCandidate[3],
						bestCandidate[4],
						bestCandidate[5],
						bestCandidate[6],
						data.getBestCandidateFitness());
				cbe.setWeights(bestCandidate);
				System.out.println("FITNESS OF BEST " + cbe.getFitnessOfCaseBaseUsingLeaveOneOut());
			}
		});
		
		BitString result = engine.evolve(500,25, new TargetFitness(0.90, true));
		System.out.println(result);
		
	}
	public static double[] makeWeightsFromBitString(BitString bitString) {
		double[] featureWeights = new double[7];
		int counter = 0;
		int numberOfBitsInAWeight =bitString.getLength()/featureWeights.length; 
		BitString tempBitString = new BitString(numberOfBitsInAWeight);
		for (int i=0; i<featureWeights.length;i++) {
			for (int j=0;j< numberOfBitsInAWeight;j++) {
				tempBitString.setBit(j, bitString.getBit(counter));
				counter ++;
			}
			featureWeights[i] = Double.parseDouble(tempBitString.toNumber().toString())/127;
		}
		
		return featureWeights;
	}

}
