package main.java.genetic_algorithm;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import org.uncommons.maths.number.ConstantGenerator;
import org.uncommons.maths.number.NumberGenerator;
import org.uncommons.maths.random.Probability;
import org.uncommons.watchmaker.framework.EvolutionaryOperator;

public class DoubleArrayMutation implements EvolutionaryOperator<double[]> {
	private final NumberGenerator<Probability> mutationProbabilty;
	private final NumberGenerator<Integer> mutationCount;
	
	public DoubleArrayMutation(Probability mutationProability) {
		this(new ConstantGenerator<Probability>(mutationProability),
				new ConstantGenerator<Integer>(1));
	}
	public DoubleArrayMutation(
			ConstantGenerator<Probability> mutationProbability,
			ConstantGenerator<Integer> mutationCount) {
		this.mutationProbabilty = mutationProbability;
		this.mutationCount = mutationCount;
	}
	/**
	 * @param args
	 */

	@Override
	public List<double[]> apply(List<double[]> selectedCandidates, Random rng) {
		List<double[]> mutatedPopulation = new ArrayList<double[]>(selectedCandidates.size());
		for (double[] d : selectedCandidates)
		{
			mutatedPopulation.add(mutateDoubleArray(d, rng));
		}
		return mutatedPopulation;
	}
	
	private double[] mutateDoubleArray(double[] individual, Random rng) {
		if (mutationProbabilty.nextValue().nextEvent(rng)) 
		{
			double[] mutatedIndividual = individual;
			int mutations = mutationCount.nextValue();
			for (int i = 0; i<mutations;i++) {
				mutatedIndividual[rng.nextInt(mutatedIndividual.length-1)] = rng.nextDouble();
			}
			return mutatedIndividual;
		}
		return individual;
	}

}
