from PyANGKernel import *

class ControlPlan(object):
    def __init__(self,model,received_mcp,gk_cp):
        """
        gk_cp - Control Plan used in Aimsun
        control_junction - Control Plan data for a particular node
        """
        self.model = model
        self.received_mcp = received_mcp
        self.gk_cp = gk_cp
        self.control_junction = self.gk_cp.getControlJunction(305)
        self.gk_phases = self.control_junction.getPhases()

    def set_phases(self):
        total_time = 0.0
#        print "set phases"
        print "len gk phases" , len(self.gk_phases)
        for i in xrange(len(self.gk_phases)):
            self.gk_phases[i].setFrom(total_time)
            try:
                self.gk_phases[i].setDuration(self.received_mcp[i])
                print "received mcp ", self.received_mcp[i]
            except:
                print "Different number of phases in aimsun signal plan vs signal plan object, setting lenght as 0"
                self.gk_phases[i].setDuration(0)
            total_time += self.gk_phases[i].getDuration()
            self.control_junction.setCycle(total_time)
            
    
class ControlPlanFactory:
    @staticmethod
    def get_control_plans_used_in_model(model):
        control_plan = model.getType( "GKControlPlan" )
        control_plans = []
        for types in model.getCatalog().getUsedSubTypesFromType( control_plan ):
            for cp in types.itervalues():
                control_plans.append(cp)
        return control_plans

    @staticmethod
    def create_control_plan(model,phases,gk_cp):
        return ControlPlan(model,phases,gk_cp)

    @staticmethod
    def get_dynamic_master_control_plan(model):
        return get_master_control_plan_by_name(model,"dynamic_mp")
    
    @staticmethod
    def get_static_master_control_plan(model):
        return get_master_control_plan_by_name(model,"static_mp")

def get_master_control_plan_by_name(model,name):
    m = model.getType("GKMasterControlPlan")
    for types in model.getCatalog().getUsedSubTypesFromType(m):
        for mcp in types.itervalues():
            if (mcp.getName() == name):
#                print "Found matching name"
                return mcp
    print "Did not find a MasterControlPlan with the name: ", name
    return None
