from PyANGBasic import *
from PyANGKernel import *
from PyANGGui import *
from PyANGConsole import *
from PyANGAimsun import *

from control_plan import ControlPlanFactory as CPF
from database_manager import DatabaseManager as DM
from case import Case
from static_plan import StaticPlanFactory as SPF

import tcp_client as TCPC

N_CENTROID = 329
E_CENTROID = 326
S_CENTROID = 330
W_CENTROID = 320

STATIC_CP_3_PHASES = 0
DYNAMIC_CP_3_PHASES = 1
STATIC_CP_2_PHASES = 2
DYNAMIC_CP_2_PHASES = 3

class SimulationCycle(object):
    def __init__(self, gui, replication_id, db_file,
                 random_seed):
        self.gui = gui
        self.model = self.gui.getActiveModel()
        self.plugin = GKSystem.getSystem().getPlugin("GGetram") 
        self.simulator = None
        self.demands = get_all_demands_in_model(self.model)
        self.dmcp = CPF.get_dynamic_master_control_plan(self.model)
        self.smcp = CPF.get_static_master_control_plan(self.model)
        self.gk_cps = CPF.get_control_plans_used_in_model(self.model)
        self.replication = self.model.getCatalog().find(replication_id)
        self.experiment = self.replication.getExperiment()
        self.scenario = self.experiment.getScenario()
        self.database_manager = DM(db_file)

        self.case_generator = yield_demand_from_model(self.model)
#        print "lol"
#        self.case_generator = yield_case_from_model(self.model)
        self.cases = self.make_cases()
#        print "rofl"
        self.case = None
        self.received_mcp = []
        self.result_sets = None

        self.gk_traffic_demand = \
            get_all_demands_in_model(self.model).pop(0)
        self.replication.setRandomSeed(random_seed)
        
        self.scenario.setDemand(self.gk_traffic_demand)
                
    def run_simulation_cycle(self):
        """
        Main loop
        """
        for i in xrange(self.number_of_simulation_cycles):
            pass

    def obtain_case(self):
        """
        Obtains the traffic demand which will be used as the case. If
        no specific traffic demand is given, a generator iterates
        through all demands and returns the next demand item.
        """
        pass
      
    def apply_master_control_plan(self):
        pass

    def simulate(self):
        """
        Starts an Aimsun simulation
        """
        if self.simulator.isBusy() == False:
            if self.experiment.getSimulatorEngine()\
                    == GKExperiment.eMicro:
                self.replication.enableVehiclesInBatch(False)
                self.simulator.addSimulationTask(self.replication,
                                            GKReplication.eBatch)
                self.simulator.simulate()

    def save_result(self):
        """
        Saving the result to the local sqlite file.
        """
        self.gui.save()

    def obtain_result_set(self):
        """
        Using a database manager in order to obtain the relevant data
        from the sqlite file
        """
        self.result_sets = self.database_manager.get_data()

    def make_cases(self):
        cases = []
        """
        Scenario 1
        """
#        cases.append(Case("Fall","Monday","Night","Cloudy","Dry","Medium","None","59,57,2,2,4,3"))
        #cases.append(Case("Fall","Monday","Morning","Precipitation","Dry","Medium","None","612,854,53,34,190,140"))
        cases.append(Case("Fall","Monday","Noon","Precipitation","Wet","Medium","None","733,755,35,32,53,41"))
        #cases.append(Case("Fall","Monday","Afternoon","Precipitation","Dry","Medium","None","1153,690,115,30,130,84,39"))
        #cases.append(Case("Fall","Monday","Evening","Precipitation","Dry","Medium","None","408,366,30,22,37,12"))

        # """
        # Scenario 2 - Weekend
        # """
        #cases.append(Case("Fall","Saturday","Night","Cloudy","Dry","Medium","None","189,156,7,8,9,6"))
        #cases.append(Case("Fall","Saturday","Morning","Cloudy","Dry","Medium","None","150,147,53,1,2,12"))
#        cases.append(Case("Fall","Saturday","Noon","Sunny","Dry","Medium","None","522,727,35,10,36,12"))
        #        cases.append(Case("Fall","Saturday","Afternoon","Sunny","Dry","Medium","None","782,509,37,18,50,11"))
        #cases.append(Case("Fall","Saturday","Evening","Cloudy","Dry","Medium","None","257,282,14,6,22,4"))

        """
        Scenario 3 - Game day
        """
        #cases.append(Case("Fall","Thursday","Evening","Sunny","Dry","Medium","Football","669,589,138,65,58,16"))
        #cases.append(Case("Fall","Sunday","Evening","Cloudy","Dry","Medium","Football","514,562,137,59,48,10"))

        # """
        # Scenario 4 - Vaccation
        # """
        #cases.append(Case("Winter","Sunday","Morning","Precipitation","Wet","Medium","Vacation","142,131,2,0,7,7"))
        #cases.append(Case("Winter","Sunday","Afternoon","Precipitation","Wet","Medium","Vacation","838,500,43,45,84,25"))
        
        """
        Scenario 5 - Slippery roads
        """
        #cases.append(Case("Fall","Monday","Afternoon","Cloudy","Icy","Freezing","None","1153,690,115,30,130,84,39"))
        #cases.append(Case("Fall","Saturday","Afternoon","Sunny","Icy","Freezing","None","782,509,37,18,50,11"))
        #cases.append(Case("Fall","Monday","Noon","Cloudy","Icy","Freezing","None","733,755,35,32,53,41"))
        #cases.append(Case("Fall","Saturday","Noon","Sunny","Icy","Freezing","None","522,727,35,10,36,12"))

        print "self.generator " , self.case_generator.next()
        return cases

    def modify_gk_traffic_demand(self,inn_s,inn_n,from_e,from_w,inn_e,inn_w):
        centroids = []
        north_centroid = self.model.getCatalog().find(N_CENTROID)
        east_centroid = self.model.getCatalog().find(E_CENTROID)
        south_centroid = self.model.getCatalog().find(S_CENTROID)
        west_centroid = self.model.getCatalog().find(W_CENTROID)

        #Turn percentages on a weekday.
        TURNPES = 0.4
        TURNPEN = 0.55
        TURNPEW = 0.5
        TURNPWS = 0.2
        TURNPWN = 0.7
        TURNPWE = 0.1

        # #Turn percentage on a saturday.
        # TURNPES = 0.49
        # TURNPEN = 0.48
        # TURNPEW = 0.03
        # TURNPWS = 0.38
        # TURNPWN = 0.6
        # TURNPWE = 0.02

        # #Turn percentage on a sunday.
        # TURNPES = 0.52
        # TURNPEN = 0.45
        # TURNPEW = 0.03
        # TURNPWS = 0.38
        # TURNPWN = 0.6
        # TURNPWE = 0.02

        TURNPNE = 0.03
        TURNPNW = 0.01
        TURNPNS = 0.96

        TURNPSW = 0.02
        TURNPSE = 0.02
        TURNPSN = 0.96

        es = int(TURNPES * from_e)
        en = int(TURNPEN * from_e)
        ew = from_e - es - en
        ws = int(TURNPWS * from_w)
        wn = int(TURNPWN * from_w)
        we = from_w - ws - wn
        ns = inn_s - es - ws
        ne = int(TURNPNE * (ns/TURNPNS))
        sn = inn_n - en - wn
        sw = int(TURNPSW * (sn/TURNPSN))
        nw = inn_w - ew - sw
        se = inn_e - we - ne
        
        for schedule_demand_item in self.gk_traffic_demand.getSchedule():
            print "schedule demand item"
            tdi = schedule_demand_item.getTrafficDemandItem()
            tdi.setTrips(north_centroid,south_centroid,ns)
            tdi.setTrips(north_centroid,west_centroid,nw)
            tdi.setTrips(north_centroid,east_centroid,ne)

            tdi.setTrips(south_centroid,north_centroid,sn)
            tdi.setTrips(south_centroid,west_centroid,sw)
            tdi.setTrips(south_centroid,east_centroid,se)

            tdi.setTrips(east_centroid,west_centroid,ew)
            tdi.setTrips(east_centroid,north_centroid,en)
            tdi.setTrips(east_centroid,south_centroid,es)

            tdi.setTrips(west_centroid,east_centroid,we)
            tdi.setTrips(west_centroid,north_centroid,wn)
            tdi.setTrips(west_centroid,south_centroid,ws)

        print "gk_traffic_demand modified"


class CBRSimulationCycle(SimulationCycle):
    def __init__(self, gui, replication_id, db_file,
                 random_seed):
        super(CBRSimulationCycle, self).__init__(gui,
                                                 replication_id, 
                                                 db_file,
                                                 random_seed)
        self.tcp_client = TCPC.TCPClient()

    def run_simulation_cycle(self):
        """
        Main loop
        """
        self.simulator = self.plugin.getCreateSimulator(self.model)
        self.obtain_case()
        self.send_case_to_external_system()
        self.receive_mcp_from_external_system()
        self.apply_master_control_plan()
        self.simulate()
        self.save_result()
        self.obtain_result_set()
        self.send_result()

    def obtain_case(self):
        """
        Obtains the traffic demand which will be used as the case. If
        no specific traffic demand is given, a generator iterates
        through all demands and returns the next demand item.
        """
        print "obtain case"
        self.case = self.cases[0]
        print self.cases[0]
        self.modify_gk_traffic_demand(self.case.ns_demand,
                                      self.case.sn_demand,
                                      self.case.ew_demand,
                                      self.case.we_demand,
                                      self.case.ne_demand,
                                      self.case.sw_demand)
        
    def adapt_case_for_external_system(self):
        print "send case to external system"
        print "self.case " , self.case
        return str(self.case)

    def send_case_to_external_system(self):
        self.tcp_client.send_data(self.adapt_case_for_external_system())

    def receive_mcp_from_external_system(self):
        """
        Takes the data received master control plan from the client
        and formats the data from "1,2,3,4-5,6,7,8" to a list(mcp)
        consisting of lists(cp's) with int values(phase lengths)
        [[1,2,3,4],[5,6,7,8]]
        """
        print "receive mcp from external system"
        self.received_mcp = []
        #Removes \r\n at the end of line
        received_data = self.tcp_client.receive_data()[:-2]
        received_data = received_data.split('-')
        for cp in received_data:
            cp = cp.split(',')
            temp_phase = []
            for phase_time in cp:
                temp_phase.append(int(phase_time))
            self.received_mcp.append(temp_phase)
            print "received_mcp ", self.received_mcp

      
    def apply_master_control_plan(self):
        print "apply master control plan"
        schedules = self.make_cp_schedules(self.received_mcp)
        self.scenario.setMasterControlPlan(self.dmcp)
#        self.dmcp.removeSchedule()
        print "GLEEN ", len(self.gk_cps)
        for i in xrange(len(self.received_mcp)):
            print "length of cp " , len(self.received_mcp[i])
            if len(self.received_mcp[i]) == 4:
                cp = CPF.create_control_plan(self.model,
                                             self.received_mcp[i],
                                             self.gk_cps[DYNAMIC_CP_2_PHASES])
            else:
                cp = CPF.create_control_plan(self.model,
                                             self.received_mcp[i],
                                             self.gk_cps[DYNAMIC_CP_3_PHASES])
            cp.set_phases()
            schedules[i].setControlPlan(cp.gk_cp)
            self.dmcp.addToSchedule(schedules[i])

    def make_cp_schedules(self,received_mcp):
        """
        Adds the received master control plan to schedule
        """
        schedules = []
        for i in xrange(len(received_mcp)):
            schedule = GKScheduleMasterControlPlanItem()
            schedule.setFrom((3600/len(received_mcp))*i)
            schedule.setDuration(3600/len(received_mcp))
            schedules.append(schedule)
        return schedules

    def simulate(self):
        """
        Starts an Aimsun simulation
        """
        if self.simulator.isBusy() == False:
            if self.experiment.getSimulatorEngine()\
                    == GKExperiment.eMicro:
                self.replication.enableVehiclesInBatch(False)
                self.simulator.addSimulationTask(self.replication,
                                            GKReplication.eBatch)
                self.simulator.simulate()

    def save_result(self):
        """
        Saving the result to the local sqlite file.
        """
        self.gui.save()

    def obtain_result_set(self):
        """
        Using a database manager in order to obtain the relevant data
        from the sqlite file
        """
        self.result_sets = self.database_manager.get_data()

    def send_result(self):
        """
        Sending the relevant information obtained by the database
        manager
        """
        result = self.database_manager.get_data()
        self.tcp_client.send_data(result[len(result)-1])


class StaticPlanSimulationCycle(SimulationCycle):
    def __init__(self, gui, replication_id, db_file,
                 random_seed): 
        super(StaticPlanSimulationCycle, self).__init__(gui, 
                                                        replication_id, 
                                                        db_file,
                                                        random_seed)

        static_plan_factory = SPF("sm/static_plans.txt")
        self.static_plans = static_plan_factory.make_static_plans_from_file()

    def run_simulation_cycle(self):
        """
        Main loop
        """
        self.simulator = self.plugin.getCreateSimulator(self.model)
        self.obtain_case()
        self.find_static_plan_for_case()
        self.apply_master_control_plan()
        self.simulate()
        self.save_result()
        self.obtain_result_set()

    def obtain_case(self):
        """
        Obtains the traffic demand which will be used as the case. If
        no specific traffic demand is given, a generator iterates
        through all demands and returns the next demand item.
        """
        print "obtain case"
        self.case = self.cases[0]
        print self.cases[0]
        self.modify_gk_traffic_demand(self.case.ns_demand,self.case.sn_demand,
        self.case.ew_demand, self.case.we_demand,self.case.ne_demand,self.case.sw_demand)

    def find_static_plan_for_case(self):
        mcp = []
        for sp in self.static_plans:
            if sp.time_of_day == self.case.time_of_day:
                if sp.day_of_week == 'Saturday' and self.case.is_saturday():
                    mcp.append(sp.plan)
                elif sp.day_of_week == 'Sunday' and self.case.is_sunday():
                    mcp.append(sp.plan)
                elif sp.day_of_week == 'Weekday' and self.case.is_weekday():
                    mcp.append(sp.plan)

        self.received_mcp = mcp
      
    def apply_master_control_plan(self):
        print "apply master control plan"
        schedules = self.make_cp_schedules(self.received_mcp)
        self.scenario.setMasterControlPlan(self.smcp)
#        self.smcp.removeSchedule()
        for i in xrange(len(self.received_mcp)):
            print "length of cp " , len(self.received_mcp[i])
            if len(self.received_mcp[i])==4:
                cp = CPF.create_control_plan(self.model,
                                             self.received_mcp[i],
                                             self.gk_cps[STATIC_CP_2_PHASES])
                                  
            else:
                cp = CPF.create_control_plan(self.model,
                                             self.received_mcp[i],
                                             self.gk_cps[STATIC_CP_3_PHASES])

            cp.set_phases()
            schedules[i].setControlPlan(cp.gk_cp)
            self.dmcp.addToSchedule(schedules[i])

    def make_cp_schedules(self,received_mcp):
        """
        Adds the received master control plan to schedule
        """
        schedules = []
        for i in xrange(len(received_mcp)):
            schedule = GKScheduleMasterControlPlanItem()
            schedule.setFrom((3600/len(received_mcp))*i)
            schedule.setDuration(3600/len(received_mcp))
            schedules.append(schedule)
        return schedules

    def simulate(self):
        """
        Starts an Aimsun simulation
        """
        if self.simulator.isBusy() == False:
            if self.experiment.getSimulatorEngine()\
                    == GKExperiment.eMicro:
                self.replication.enableVehiclesInBatch(False)
                self.simulator.addSimulationTask(self.replication,
                                            GKReplication.eBatch)
                self.simulator.simulate()

    def save_result(self):
        """
        Saving the result to the local sqlite file.
        """
        self.gui.save()

    def obtain_result_set(self):
        """
        Using a database manager in order to obtain the relevant data
        from the sqlite file
        """
        self.result_sets = self.database_manager.get_data()
        print self.result_sets[len(self.result_sets)-1]

def yield_demand_from_model(model):
    demands = get_all_demands_in_model(model)
    i = -1
    while(True):
        i += 1
        print "i" , i
        yield demands[i%len(demands)]

def get_all_demands_in_model(model):
    demands = []
    demand = model.getType("GKTrafficDemand")
    for types in model.getCatalog().getUsedSubTypesFromType(demand):
        for m in types.itervalues():
            demands.append(m)
    return demands
