import sys
import random
from PyANGBasic import *
from PyANGKernel import *
from PyANGGui import *
from PyANGConsole import *
from PyANGAimsun import *

from simulation_cycle import CBRSimulationCycle as CBRSC
from simulation_cycle import StaticPlanSimulationCycle as SPSC

from database_manager import DatabaseManager as DM
from control_plan import ControlPlanFactory as CPF

from case import Case


ELGSETER_REPL = 35802
STATIC_REPL = 370
CBR_REPL = 467

class SimulatorManager(object):
    def __init__(self, gui, number_of_simulation_cycles=1,
                 db_file = "test.sqlite",
                 traffic_demand_to_use = 2):
        self.gui = gui
        self.model = self.gui.getActiveModel()
        self.plugin = GKSystem.getSystem().getPlugin("GGetram") 
        self.simulator = None
        self.number_of_simulation_cycles = number_of_simulation_cycles
        self.dmcp = CPF.get_dynamic_master_control_plan(self.model)
        self.smcp = CPF.get_static_master_control_plan(self.model)
        self.gk_cps = CPF.get_control_plans_used_in_model(self.model)
        self.database_manager = DM(db_file)
        self.db_file = db_file

    def run_simulation_cycles(self):
        for i in xrange(self.number_of_simulation_cycles):
#            random_seed = self.seed_generator()
            random_seed = 1234567890
                        
            static_cycle = SPSC(self.gui, 
                                STATIC_REPL, self.db_file, 
                                random_seed)
            static_cycle.run_simulation_cycle()
            static_result = static_cycle.result_sets[len(static_cycle.result_sets)-1]


            cbr_cycle = CBRSC(self.gui, 
                              CBR_REPL, self.db_file, 
                              random_seed)
            cbr_cycle.run_simulation_cycle()
            cbr_result = cbr_cycle.result_sets[len(cbr_cycle.result_sets)-1]

            print "Static result", static_result            
            print "CBR result", cbr_result


    def seed_generator(self):
        """
        Seed used for generating traffic in simulations
        """
        return random.randint(0,sys.maxint)
